package functional.set.subset;

final class Set {
	/*@ nullable @*/int[] items;
	/*@ nullable @*/ Set res;

	/*@ 
	  @ requires this.items != null;
	  @ requires (\forall int i, j; 0<=i && i<this.items.length && 0<=j && j<this.items.length && i!=j; this.items[i] != this.items[j]);
	  @ requires 0 <= this.items.length;
	  @ requires p!=null && p.items != null;
	  @ requires (\forall int i, j; 0<=i && i<p.items.length && 0<=j && j<p.items.length && i!=j; p.items[i] != p.items[j]);
	  @ requires 0 <= p.items.length;
	  @ 
	  @ ensures (\forall int i; 0<=i&&i<res.items.length;(\exists int j; 0<=j&&j<items.length; res.items[i]==items[j]));
	  @ assignable \everything;
	  @ diverges true;
	  @*/
	void wrapper(/*@ nullable @*/ Set p) {
		intersect(p);
	}

	void intersect(/*@ nullable @*/ Set p) {
		res = new Set();
		res.items = new int[0];
		int len = this.items.length;
		int index = 0;
		/*@ loop_invariant 0 <= index && index <= this.items.length;
	      @ loop_invariant res.items != null && res.items.length >= 0;
	      @ loop_invariant this.items != null && this.items.length >= 0;
	      @ loop_invariant p.items != null && p.items.length >= 0;
	      @ loop_invariant (\forall int k; 0<=k&&k<res.items.length;(\exists int j; 0<=j&&j<items.length; res.items[k]==items[j]));
		  @ assignable res.items;
		  @*/
		while (index < len) {
			int v = this.items[index];
			boolean found = false;
			int pLen = p.items.length;
			int pIndex = 0;
			/*@ loop_invariant (\forall int j; 0<=j && j<pIndex; this.items[pIndex] != v);
			  @ loop_invariant 0<=pIndex && pIndex<=this.items.length;
			  @ loop_invariant this.items!= null;
			  @ loop_invariant this.items.length >= 0;
			  @ assignable \strictly_nothing;
			  @*/
			while (pIndex < pLen) {
				if (p.items[pIndex] == v) {
					found = true;
					break;

				}
				pIndex = pIndex + 1;
			}

			if (found) {
				boolean resFound = false;
				int resLenF = res.items.length;
				int resIndexF = 0;
				/*@ loop_invariant (\forall int j; 0<=j && j<resIndexF; res.items[j] != v);
				  @ loop_invariant 0<=resIndexF && resIndexF<=res.items.length;
				  @ loop_invariant res.items!= null;
				  @ loop_invariant res.items.length >= 0;
				  @ assignable \strictly_nothing;
				  @*/
				while (resIndexF < resLenF) {
					if (res.items[resIndexF] == v) {
						resFound = true;
						break;
					}
					resIndexF = resIndexF + 1;
				}

				if (!resFound) {
					int resLen = res.items.length;

					int[] newList = new int[resLen + 1];
					int resIndex = 0;
					/*@ loop_invariant (\forall int j; 0<=j && j<resIndex; newList[j] == res.items[j]);
					  @ loop_invariant res.items != null;
					  @ loop_invariant res.items.length >= 0;
					  @ loop_invariant 0<=resIndex && resIndex <= res.items.length;
					  @ assignable newList[*];
					  @*/
					while (resIndex < resLen) {
						newList[resIndex] = res.items[resIndex];
						resIndex = resIndex + 1;
					}
					newList[resIndex] = v;
					res.items = newList;
				}
			}
			index = index + 1;
		}
	}
}
