package abstraction.dictionary.ArrayIndexOutOfBoundsException;

import abstraction.RuntimeException;

final class Dictionary {
	/*@ nullable @*/Key[] keys;
	/*@ nullable @*/Value[] values;

	/*@ requires !RuntimeException.NullPointerException;
	  @ requires !RuntimeException.ArrayIndexOutOfBoundsException;
	  @ requires !RuntimeException.NegativeArraySizeException;
	  @ requires keys != null && values != null;
	  @ requires (\forall int i,j; 0<=i && i<j && j<keys.length; keys[i] != keys[j]);
	  @ requires keys.length == values.length;
	  @ requires 0<=keys.length && keys.length<Integer.MAX_VALUE;
	  @ ensures !RuntimeException.ArrayIndexOutOfBoundsException;
	  @ diverges true;
	  @ assignable \everything;
	  @*/
	void wrapper(/*@ nullable @*/ Key k, /*@ nullable @*/ Value v) {
		put(k, v);
	}

	void put(/*@ nullable @*/ Key k, /*@ nullable @*/ Value v) {
		int pos = -1;
		int i = 0;
		int keyLen = this.keys.length;
		/*@ 
		  @ maintaining i>=0 && i<= keyLen;
          @ maintaining pos<=i && pos<keyLen;
		  @ assignable \strictly_nothing;
		 */
		while (i < keyLen) {
			if (i < 0 || i >= keyLen) {
				RuntimeException.ArrayIndexOutOfBoundsException = true;
				return;
			}
			if ( pure_boolean() ) { //			if (this.keys[i] == k) {
				pos = i;
				break;
			}
			i = i + 1;
		}

		if (pos >= 0) {
			int vLen = this.values.length;
			if (pos < 0 || pos >= vLen) {
				RuntimeException.ArrayIndexOutOfBoundsException = true;
				return;
			}
			this.values[pos] = pure_Value(); //			this.values[pos] = v;
		} else {
			Key[] oldKs = this.keys;
			int oldKLen = oldKs.length;
			int iK = 0;
			this.keys = new Key[oldKLen + 1];
			if (oldKLen < 0 || oldKLen >= this.keys.length) {
				RuntimeException.ArrayIndexOutOfBoundsException = true;
				return;
			}
			this.keys[oldKLen] = pure_Key(); //			this.keys[oldKLen] = k;
			/*@ 
			  @ maintaining iK>=0 && iK<=oldKs.length;
			  @ maintaining iK>=0 && iK<=this.keys.length;
			  @ assignable this.keys[*];
			 */
			while (iK < oldKLen) {
				if (iK < 0 || iK >= oldKs.length) {
					RuntimeException.ArrayIndexOutOfBoundsException = true;
					return;
				}
				this.keys[iK] = pure_Key(); //				this.keys[iK] = oldKs[iK];
				iK = iK + 1;
			}

			Value[] oldVs = this.values;
			int oldVLen = oldVs.length;
			int iV = 0;
			this.values = new Value[oldVLen + 1];
			if (oldVLen < 0 || oldVLen >= this.values.length) {
				RuntimeException.ArrayIndexOutOfBoundsException = true;
				return;
			}
			this.values[oldVLen] = pure_Value(); //			this.values[oldVLen] = v;
			/*@ 
			  @ maintaining iV>=0 && iV<=oldVs.length;
			  @ maintaining iV>=0 && iV<=this.values.length;
			  @ assignable this.values[*];
			 */
			while (iV < oldVLen) {
				if (iV < 0 || iV >= oldVs.length) {
					RuntimeException.ArrayIndexOutOfBoundsException = true;
					return;
				}
				if (iV < 0 || iV >= this.values.length) {
					RuntimeException.ArrayIndexOutOfBoundsException = true;
					return;
				}
				this.values[iV] = pure_Value(); //				this.values[iV] = oldVs[iV];
				iV = iV + 1;
			}
		}
	}
	/*@ assignable \strictly_nothing; @*/
	native boolean /*@ helper @*/ pure_boolean();
	/*@ assignable \strictly_nothing; @*/
	native void /*@ helper @*/ pure_void();
	/*@ assignable \strictly_nothing; @*/
	native Key /*@ helper @*/ pure_Key();
	/*@ assignable \strictly_nothing; @*/
	native Value /*@ helper @*/ pure_Value();
}
