package abstraction.set.ArrayIndexOutOfBoundsException;

import abstraction.RuntimeException;

final class Set {
	/*@ nullable @*/int[] items;
	/*@ nullable @*/ Set res;

	/*@ requires !RuntimeException.NullPointerException;
	  @ requires !RuntimeException.ArrayIndexOutOfBoundsException;
	  @ requires !RuntimeException.NegativeArraySizeException;
	  @ requires this.items != null;
	  @ requires (\forall int i; 0<=i && i< this.items.length; this.items[i] != null);
	  @ requires (\forall int i, j; 0<=i && i<this.items.length && 0<=j && j<this.items.length && i!=j; this.items[i] != this.items[j]);
	  @ requires 0 <= this.items.length;
	  @ requires p!=null && p.items != null;
	  @ requires (\forall int i; 0<=i && i< p.items.length; p.items[i] != null);
	  @ requires (\forall int i, j; 0<=i && i<p.items.length && 0<=j && j<p.items.length && i!=j; p.items[i] != p.items[j]);
	  @ requires 0 <= p.items.length;
	  @ ensures !RuntimeException.ArrayIndexOutOfBoundsException;
	  @ assignable \everything;
	  @ diverges true;
	  @*/
	void wrapper(/*@ nullable @*/ Set p){
		intersect(p);
	}

	void intersect(/*@ nullable @*/ Set p) {
		res = new Set();
		res.items = new int[0];
		int len = this.items.length;
		int index = 0;
                //@ loop_invariant 0 <= index && index <= len && len == items.length;
                //@ assignable res.items, res.items[*];
		while (index < len) {
			if (index < 0 || index >= len) {
				RuntimeException.ArrayIndexOutOfBoundsException = true;
				return;
			}
			int v = this.items[index];
			boolean found = false;
			int pLen = p.items.length;
			int pIndex = 0;
                        //@ loop_invariant 0 <= pIndex && pIndex <= pLen && pLen == p.items.length;
                        //@ assignable \nothing;
			while (pIndex < pLen) {
				if (pIndex < 0 || pIndex >= pLen) {
					RuntimeException.ArrayIndexOutOfBoundsException = true;
					return;
				}
				if (p.items[pIndex] == v) {
					found = true;
					break;

				}
				pIndex = pIndex + 1;
			}

			if (found) {
				boolean resFound = false;
				int resLenF = res.items.length;
				int resIndexF = 0;
                                //@ loop_invariant 0 <= resIndexF && resIndexF <= resLenF && resLenF == res.items.length;
                                //@ assignable \strictly_nothing;
				while (resIndexF < resLenF) {
					if (resIndexF < 0 || resIndexF >= res.items.length) {
						RuntimeException.ArrayIndexOutOfBoundsException = true;
						return;
					}
					if (res.items[resIndexF] == v) {
						resFound = true;
						break;
					}
					resIndexF = resIndexF + 1;
				}

				if (!resFound) {
					int resLen = res.items.length;

					int[] newList = new int[resLen + 1];
					int resIndex = 0;
                                        //@ loop_invariant 0 <= resIndex && resIndex <= resLen && resLen == res.items.length;
                                        //@ assignable newList[*];
					while (resIndex < resLen) {
						if (resIndex < 0 || resIndex >= res.items.length) {
							RuntimeException.ArrayIndexOutOfBoundsException = true;
							return;
						}
						if (resIndex < 0 || resIndex >= newList.length) {
							RuntimeException.ArrayIndexOutOfBoundsException = true;
							return;
						}
						newList[resIndex] = res.items[resIndex];
						resIndex = resIndex + 1;
					}
					if (resIndex < 0 || resIndex >= newList.length) {
						RuntimeException.ArrayIndexOutOfBoundsException = true;
						return;
					}
					newList[resIndex] = v;
					res.items = newList;
				}
			}
			index = index + 1;
		}
	}
}
